@@ -27,6 +27,9 @@ class TestSession(testtools.TestCase):
         self.assertEqual(
             "http://127.0.0.1:9292/v1",
             session.parse_url(filt, "http://127.0.0.1:9292"))
+        self.assertEqual(
+            "http://127.0.0.1:9292/foo/v1",
+            session.parse_url(filt, "http://127.0.0.1:9292/foo"))
         self.assertEqual(
             "http://127.0.0.1:9292/v2",
             session.parse_url(filt, "http://127.0.0.1:9292/v2.0"))
